/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

interface nsIDOMCSSRule;
interface nsIDOMCSSStyleSheet;

[scriptable, builtinclass, uuid(36098d39-b471-47e9-976e-33fee3d81467)]
interface nsIDOMStyleRuleChangeEvent : nsIDOMEvent
{
  readonly attribute nsIDOMCSSStyleSheet stylesheet;
  readonly attribute nsIDOMCSSRule rule;
  [noscript] void initStyleRuleChangeEvent(in DOMString aTypeArg,
                                           in boolean aCanBubbleArg,
                                           in boolean aCancelableArg,
                                           in nsIDOMCSSStyleSheet aStyleSheet,
                                           in nsIDOMCSSRule aRule);
};

dictionary StyleRuleChangeEventInit : EventInit
{
  nsIDOMCSSStyleSheet stylesheet;
  nsIDOMCSSRule rule;
};

