/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_InterAppConnectionBinding_h__
#define mozilla_dom_InterAppConnectionBinding_h__

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/BindingUtils.h"
#include "mozilla/dom/CallbackInterface.h"

namespace mozilla {
namespace dom {

class MozInterAppConnection;
class NativePropertyHooks;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

namespace MozInterAppConnectionBinding {

  typedef mozilla::dom::MozInterAppConnection NativeType;

  // We declare this as an array so that retrieving a pointer to this
  // binding's property hooks only requires compile/link-time resolvable
  // address arithmetic.  Declaring it as a pointer instead would require
  // doing a run-time load to fetch a pointer to this binding's property
  // hooks.  And then structures which embedded a pointer to this structure
  // would require a run-time load for proper initialization, which would
  // then induce static constructors.  Lots of static constructors.
  extern const NativePropertyHooks sNativePropertyHooks[];
  JSObject*
  DefineDOMInterface(JSContext* aCx, JS::Handle<JSObject*> aGlobal, JS::Handle<jsid> id, bool aDefineOnGlobal);

  bool
  ConstructorEnabled(JSContext* cx, JS::Handle<JSObject*> obj);

  const JSClass*
  GetJSClass();

  JSObject*
  Wrap(JSContext* aCx, JS::Handle<JSObject*> aScope, mozilla::dom::MozInterAppConnection* aObject, nsWrapperCache* aCache);

  template <class T>
  inline JSObject* Wrap(JSContext* aCx, JS::Handle<JSObject*> aScope, T* aObject)
  {
    return Wrap(aCx, aScope, aObject, aObject);
  }

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, JS::Heap<JSObject*>* aProtoAndIfaceArray, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

  JS::Handle<JSObject*>
  GetConstructorObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal, bool aDefineOnGlobal = true);

} // namespace MozInterAppConnectionBinding



class MozInterAppConnectionJSImpl : public CallbackInterface
{
public:
  explicit inline MozInterAppConnectionJSImpl(JSObject* aCallback)
    : CallbackInterface(aCallback)
  {
  }

  void Cancel(JSCompartment* aCompartment, ErrorResult& aRv);

  void __Init(JSCompartment* aCompartment, const nsAString& keyword, const nsAString& publisher, const nsAString& subsriber, ErrorResult& aRv);

  void GetKeyword(JSCompartment* aCompartment, nsString& retval, ErrorResult& aRv);

  void GetPublisher(JSCompartment* aCompartment, nsString& retval, ErrorResult& aRv);

  void GetSubscriber(JSCompartment* aCompartment, nsString& retval, ErrorResult& aRv);
};


class MozInterAppConnection MOZ_FINAL : public nsISupports,
                                        public nsWrapperCache
{
public:
  NS_DECL_CYCLE_COLLECTING_ISUPPORTS
  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(MozInterAppConnection)

private:
  nsRefPtr<MozInterAppConnectionJSImpl> mImpl;
  nsCOMPtr<nsISupports> mParent;

public:
  MozInterAppConnection(JS::Handle<JSObject*> aJSImplObject, nsPIDOMWindow* aParent);

  nsISupports* GetParentObject() const;

  virtual JSObject* WrapObject(JSContext* aCx, JS::Handle<JSObject*> aScope) MOZ_OVERRIDE;

  static already_AddRefed<MozInterAppConnection> Constructor(const GlobalObject& global, JSContext* cx, const nsAString& keyword, const nsAString& publisher, const nsAString& subsriber, ErrorResult& aRv);

  void GetKeyword(JSCompartment* aCompartment, nsString& retval, ErrorResult& aRv) const;

  void GetPublisher(JSCompartment* aCompartment, nsString& retval, ErrorResult& aRv) const;

  void GetSubscriber(JSCompartment* aCompartment, nsString& retval, ErrorResult& aRv) const;

  void Cancel(JSCompartment* aCompartment, ErrorResult& aRv);

  static bool
  _Create(JSContext* cx, unsigned argc, JS::Value* vp);
};


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_InterAppConnectionBinding_h__
