/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


#include "domstubs.idl"

[scriptable, uuid(B31702D0-6DAC-4FA0-B93B-F043E71C8F9A)]
interface nsIDOMGeoPositionCoords : nsISupports
{
  readonly attribute double latitude;
  readonly attribute double longitude;
  readonly attribute double altitude;
  readonly attribute double accuracy;
  readonly attribute double altitudeAccuracy;
  readonly attribute double heading;
  readonly attribute double speed;
};
